const { join } = require('path')
const fs = require('fs-extra')
const inquirer = require('inquirer')
const { info, success, error } = require('../utils/tip')
const { shell } = require('execa')
const getSpinner = require('../utils/getSpinner')
const { setPkgJson } = require('../utils/fire')
const install = require('../utils/install')
module.exports = async function() {
  let projectName = process.argv[3] || ''
  if (!projectName.trim()) {
    const { _projectName } = await inquirer.prompt([
      {
        type: 'input',
        message: 'What project do you want to create：',
        name: '_projectName',
        validate(val) {
          return val.trim() ? true : 'you must provide a project name'
        }
      }
    ])
    projectName = _projectName
  }
  await createProject(projectName)
}
async function createProject(projectName) {
  const cwd = process.cwd()
  const projectPath = join(cwd, projectName)
  let generateSpinner = getSpinner('Creating... ')
  try {
    info(`\n start to create project '${projectName}',wait for a moment`)
    generateSpinner.start()
    await shell(
      `git clone git@git.2dfire-inc.com:danfan/vue-boilerplate.git ${projectName}`
    )
    await fs.remove(join(projectPath, '.git'))
    setPkgJson(projectName)
    await install(projectPath)
    generateSpinner.stop()
    success('\n √ create completed!')
    success(`\n cd ${projectName}`)
    success('\n npm start or yarn start')
  } catch (e) {
    generateSpinner.stop()
    error(e)
  }
}
