/*
将一个不是由fire-cli创建的项目初始化成可以由fire管理的项目，
 */
const {
  __pkgJson,
  __src,
  __page,
  __root,
  __nodeModules,
  __base,
  __yarnLock,
  __packageLock
} = require('../utils/paths')
const fs = require('fs-extra')
const glob = require('glob')
const { shell } = require('execa')
const { setFireConfig } = require('../utils/fire')
const { success } = require('../utils/tip')
const getSpinner = require('../utils/getSpinner')
const { join } = require('path')
const install = require('../utils/install')
async function getFireScriptsDep() {
  let { stdout } = await shell('npm view fire-scripts dependencies')
  stdout = stdout.trim()
  return stdout
    .substring(1, stdout.length - 1)
    .split(',')
    .map(item => {
      let key = item.substring(0, item.indexOf(':')).trim()
      if (key.charAt(0) === "'" || key.charAt(0) === '"') {
        key = key.substring(1, key.length - 1)
      }
      return key
    })
}
async function getFireScriptsVer() {
  const { stdout } = await shell('npm view fire-scripts version')
  return stdout
}
function createExample() {
  let exampleName = 'example'
  if (fs.pathExistsSync(join(__page, 'example.html'))) {
    exampleName = 'fire-example'
  }
  fs.copySync(
    join(__dirname, '../example/index.html'),
    join(__page, exampleName + '.html')
  )
  fs.copySync(join(__dirname, '../example/src'), join(__src, exampleName))
  setFireConfig(exampleName)
}
function existInSrcButNotPage(name) {
  return (
    !fs.pathExistsSync(join(__page, name + '.html')) &&
    fs.pathExistsSync(join(__src, name))
  )
}
function createBase() {
  if (!fs.pathExistsSync(__base)) {
    if (existInSrcButNotPage('base')) {
      // fs.copySync(join(__src, 'base'), __base)
    } else {
      fs.copySync(join(__dirname, '../example/static/__BASE__'), __base)
    }
  }
}
function createStaticAtRoot(name, isOverride = true) {
  if (isOverride || !fs.pathExistsSync(join(__root, name))) {
    fs.copySync(
      join(__dirname, `../example/static/${name}`),
      join(__root, name)
    )
  }
}
function copyStatic() {
  if (
    !fs.pathExistsSync(join(__root, 'static')) &&
    existInSrcButNotPage('static')
  ) {
    fs.copySync(join(__src, 'static'), join(__root, 'static'))
    fs.removeSync(join(__src, 'static'))
  }
}
function createTpl() {
  createStaticAtRoot('.gitignore', false)
  createStaticAtRoot('appinfo')
  createStaticAtRoot('build.sh')
  createStaticAtRoot('.prettierrc.js', false)
  createBase()
  createExample()
}
function backupProject() {
  const date = new Date()
  const y = date.getFullYear()
  const m = date.getMonth() + 1
  const d = date.getDate()
  const h = date.getHours()
  const M = date.getMinutes()
  const s = date.getSeconds()

  const projectCopy = `project-${y}${m}${d}${h}${M}${s}`
  fs.copySync(__src, join(__root, projectCopy, 'src'))
  fs.copySync(__pkgJson, join(__root, projectCopy, 'package.json'))
  fs.copySync(__page, join(__root, projectCopy, 'page'))
  fs.copySync(join(__root, 'build.sh'), join(__root, projectCopy, 'build.sh'))
}
/*
为了兼容老项目中用module.export和require('xx')
 */
async function transform() {
  const isNodeModule = name => {
    const alias = ['src', 'base', '.']
    if (alias.some(i => name.startsWith(i))) {
      return false
    }
    if (name.startsWith('apiConfig')) {
      return true
    }
    return ['', '.js'].some(item =>
      fs.pathExistsSync(join(__nodeModules, name + item))
    )
  }
  const isAlias = key => {
    return ['base', 'src', 'apiConfig'].some(i => key.startsWith(i))
  }
  const getAlias = key => {
    if (key.startsWith('src')) {
      return join(__root, key)
    }
    if (key.startsWith('base')) {
      return join(__src, key)
    }
    return null
  }
  const isImage = key => {
    return ['png', 'jpg', 'jpeg', 'gif', 'svg'].some(i => key.endsWith('.' + i))
  }
  const getRequireKeyVal = w => {
    w = w.trim()
    const name = w.substring(w.indexOf(' '), w.indexOf('=')).trim()
    let key = w.substring(w.indexOf('(') + 1, w.lastIndexOf(')')).trim()
    const at = key.charAt(0)
    if (at === "'" || at === '"') {
      key = key.substring(1, key.length - 1)
    }
    return { key, name }
  }
  const varRequireReg = /(var|let|const)[^=]*=\s*require\(.+?\)(?!.default)/g
  const moduleExportsReg = /module.exports\s*=\s*/g
  const requireReg = /require\(.+?\)(?!.default)/g
  const importReg = /import\s*{[^'"]*}\s*from\s*(('.+?')|(".+?"))/g
  const exportDefaultReg = /export\s*default\s*/g
  let flag = false

  /*
  所有用require的方式引用nodeModules中的模块的代码转为import方式
   */
  const $require2Import = function(res) {
    if (varRequireReg.test(res)) {
      flag = true
      return res.replace(varRequireReg, w => {
        const { name, key } = getRequireKeyVal(w)
        if (isNodeModule(key)) {
          return `import ${name} from '${key}'`
        }
        return '\n' + w
      })
    }
    return res
  }
  //所有的require转为require().default
  const $require2Default = function(res) {
    if (requireReg.test(res)) {
      flag = true
      return res.replace(requireReg, w => {
        const { key } = getRequireKeyVal(w)
        if (isImage(key)) {
          return w
        }
        return `${w}.default`
      })
    }
    return res
  }
  /*
  分析所有的import {xx}是否需要转为require().default
   */
  const $Import2RequireDefault = function(res, path) {
    if (importReg.test(res)) {
      flag = true
      return res.replace(importReg, w => {
        w = w.trim()
        const name = w.substring(w.indexOf(' '), w.indexOf('from')).trim()
        let cutOff = w.indexOf("'") > 0 ? "'" : '"'
        const key = w
          .substring(w.indexOf(`${cutOff}`) + 1, w.lastIndexOf(`${cutOff}`))
          .trim()
        if (isNodeModule(key)) {
          return w
        } else {
          try {
            //分析导入的文件中是否存在module.exports=
            let importPath = join(path, '../', key)
            if (isAlias(key)) {
              importPath = getAlias(key)
            }
            if (fs.pathExistsSync(importPath + '.js')) {
              importPath = importPath + '.js'
            } else if (fs.pathExistsSync(join(importPath, 'index.js'))) {
              importPath = join(importPath, 'index.js')
            }
            //获取依赖的内容
            const info = fs.readFileSync(importPath, 'utf8')
            if (moduleExportsReg.test(info)) {
              return `const ${name} = require('${key}').default`
            } else {
              return w
            }
          } catch (e) {
            console.log('\n', e, 11)
            return w
          }
        }
      })
    }
    return res
  }
  //所有的module.exports=转为export default
  const $module2ExportDefault = function(res) {
    if (moduleExportsReg.test(res)) {
      flag = true
      return res.replace(moduleExportsReg, () => {
        return 'export default '
      })
    }
    return res
  }
  const $exportDefault2Module = function(res, path) {
    if (
      ['daily.js', 'pre.js', 'publish.js', 'dev.js'].some(i =>
        path.endsWith(i)
      ) &&
      exportDefaultReg.test(res)
    ) {
      flag = true
      return res.replace(exportDefaultReg, () => {
        return 'module.exports= '
      })
    }
    return res
  }
  const srcArr = glob.sync('src/**/*.{js,vue}', { cwd: join(__root) })
  for (let i = 0, len = srcArr.length; i < len; i++) {
    const path = join(__root, srcArr[i])
    let res = await fs.readFile(path, 'utf8')
    res = res || ''
    flag = false
    res = $require2Import(res)
    res = $require2Default(res)
    res = $Import2RequireDefault(res, path)
    if (flag) {
      await fs.writeFile(path, res)
    }
  }
  /*
  将所有的除apiConfig外的module.exports=转为export default
  apiConfig中的转为module.exports=
   */
  for (let i = 0, len = srcArr.length; i < len; i++) {
    const path = join(__root, srcArr[i])
    let res = await fs.readFile(path, 'utf8')
    flag = false
    res = $module2ExportDefault(res)
    res = $exportDefault2Module(res, path)
    if (flag) {
      await fs.writeFile(path, res)
    }
  }
}
module.exports = async function() {
  const initSpinner = getSpinner('Initializing...')
  try {
    initSpinner.start()
    // backupProject()
    if (!fs.pathExistsSync(__pkgJson)) {
      await shell('npm init -f')
    }
    //获取fire-scripts的所有依赖，之后和当前项目的依赖做对比
    const allFireScriptsDep = await getFireScriptsDep()
    const currPkg = require(__pkgJson)
    const currDep = currPkg.dependencies || {}
    const currDevDep = currPkg.devDependencies || {}
    const scripts = currPkg.scripts
    if (!scripts) {
      currPkg.scripts = {
        start: 'fire-scripts start'
      }
    } else {
      scripts.start = 'fire-scripts start'
    }
    const currNewDep = {}
    const currNewDevDep = {}
    for (let name in currDep) {
      if (!allFireScriptsDep.includes(name)) {
        currNewDep[name] = currDep[name]
      }
    }
    for (let name in currDevDep) {
      if (!allFireScriptsDep.includes(name)) {
        currNewDevDep[name] = currDevDep[name]
      }
    }
    currNewDep['fire-scripts'] = await getFireScriptsVer()
    currPkg.dependencies = currNewDep
    currPkg.devDependencies = currNewDevDep
    fs.writeFileSync(__pkgJson, JSON.stringify(currPkg, null, 2))
    createTpl()
    copyStatic()
    await fs.remove(__nodeModules)
    await fs.remove(__yarnLock)
    await fs.remove(__packageLock)
    await fs.remove(join(__root, 'check_health'))
    await shell('git add .')
    await install()
    await transform()
    initSpinner.stop()
    success('\n√ Initialize completed!')
  } catch (e) {
    console.log(e)
    initSpinner.stop()
  }
}
