const fs = require('fs-extra')
const { join } = require('path')
const inquirer = require('inquirer')
const { shell, shellSync } = require('execa')
const getSpinner = require('../utils/getSpinner')
const { success, warning } = require('../utils/tip')
const { isFireProject, checkFireVersion } = require('../utils/fire')
const { __noble, __root } = require('../utils/paths')
const { page } = require('../utils/getFireConfig')()
const install = require('../utils/install')
const opn = require('opn')
const glob = require('glob')
function getProjectName() {
  let projectName = (
    require(join(process.cwd(), 'package.json')).name || ''
  ).trim()
  if (projectName.indexOf('static-') === 0) {
    projectName = projectName.substring(7)
  }
  return projectName
}
function getIndexPage() {
  return (
    (typeof page === 'string' && page.trim() === '*'
      ? glob
          .sync('./page/*.html', { cwd: __root })
          .map(name => name.slice(7, name.length - 5))
      : page || []
    ).filter(item => item !== '__BASE__')[0] || 'example'
  )
}
function isIp(ip) {
  if (typeof ip !== 'string') {
    return false
  }
  const ipArr = ip.split('.').map(i => {
    if (+i === 0) {
      return 0
    }
    return +i || -1
  })
  if (ipArr.length !== 4 || ipArr[0] < 1 || ipArr[0] > 255) {
    return false
  }
  return !ipArr.slice(1).some(item => item < 0 || item > 255)
}
//在deploy和master分支上不能用此命令
function canDeployAtCurrBranch() {
  const { stdout } = shellSync(
    `cd ${__root} && git symbolic-ref --short -q HEAD`
  )
  return stdout !== 'deploy' && stdout !== 'master'
}
module.exports = async function() {
  const nobleSpinner = getSpinner('deploying...')
  try {
    if (!isFireProject()) {
      return warning('This is not a fire project')
    }
    if (!canDeployAtCurrBranch()) {
      return warning('你不能在master或者deploy分支上进行此操作')
    }
    let { hasMerge = 'yes' } = await inquirer.prompt([
      {
        type: 'input',
        message: '部署前请先确认是否已经将最新的master代码合并过来了？',
        name: 'hasMerge',
        default: 'yes'
      }
    ])
    hasMerge = hasMerge.trim().toLowerCase()
    if (hasMerge !== 'y' && hasMerge !== 'yes') {
      return warning('请先合并代码，再执行此操作')
    }
    let serverIp = process.argv[3] || ''
    if (!serverIp.trim()) {
      let { stdout = '' } = await shell('git config user.name')
      if (stdout.indexOf('@2dfire.com') > 0) {
        stdout = stdout.substring(0, stdout.indexOf('@2dfire.com'))
      }
      const ipPath = join(__noble, stdout + '.js')
      if (fs.pathExistsSync(ipPath)) {
        serverIp = (require(ipPath) || {}).serverIp
      }
      if (!serverIp) {
        const { ip } = await inquirer.prompt([
          {
            type: 'input',
            message: 'You server ip：',
            name: 'ip',
            validate(val) {
              return val.trim() ? true : 'You must provide a server ip'
            }
          }
        ])
        serverIp = ip
      }
    }
    if (!isIp(serverIp)) {
      return warning('the ip is invalid')
    }
    nobleSpinner.start()
    const fireVer = checkFireVersion()
    if (fireVer) {
      const { latestVer } = fireVer || {}
      return warning(
        `The latest version is ${latestVer}, you must update the project config : 'fire update'`,
        true
      )
    }
    // await install(false)
    const { stdout } = await shell('git status --short')
    await shell('git add .')
    if (stdout) {
      await shell('git commit -m noble && git pull && git push origin')
    }
    await shell('node node_modules/fire-scripts/bin/index build NODE_ENV=dev')
    nobleSpinner.stop()
    const projectName = getProjectName()
    await shell(join(__dirname, `noble.sh ${serverIp} ${projectName}`))
    success(`\n√ deploy completed!`)
    opn(`http://${serverIp}/nginx/${projectName}/page/${getIndexPage()}.html`)
  } catch (e) {
    nobleSpinner.stop()
    console.log(e)
  }
}
