const fs = require('fs-extra')
const { join } = require('path')
const inquirer = require('inquirer')
const { shell } = require('execa')
const getSpinner = require('../utils/getSpinner')
const { isPageNameExists } = require('../utils/isExists')
const { success, error, warning } = require('../utils/tip')
const { isFireProject, setFireConfig } = require('../utils/fire')
const { getPageTplList, delPageTplDir } = require('../utils/pageTpl')
const getPageTplPath = require('../utils/getPageTplPath')
module.exports = async function() {
  // if (!isFireProject()) {
  //   return warning(
  //     'This is not a project that created by 2dfire-cli,so you cannot use the command'
  //   )
  // }
  let pageName = process.argv[3] || ''
  if (!pageName.trim()) {
    const { _pageName } = await inquirer.prompt([
      {
        type: 'input',
        message: 'What page do you want to create：',
        name: '_pageName',
        validate(val = '') {
          val = val.trim().toLowerCase() || 'base'
          if (['static', 'base', '__base__'].includes(val)) {
            return '你必须提供一个有效的名字'
          }
          return true
        }
      }
    ])
    pageName = _pageName
  }
  await createPage(pageName)
}
async function createPage(pageName) {
  const cwd = process.cwd()
  if (isPageNameExists(pageName)) {
    return warning(`The page ${pageName} already existed`)
  }
  const createSpinner = getSpinner('creating... ')
  createSpinner.start()
  try {
    await delPageTplDir()
    await shell(
      `git clone git@git.2dfire-inc.com:danfan/vue-page-boilerplate.git ${join(
        __dirname,
        '../template'
      )}`
    )
    const list = getPageTplList()
    createSpinner.stop()
    if (!list.length) {
      await delPageTplDir()
      error(
        "No any page template,you can create template in you local 'template' directory"
      )
    }
    let { pageTpl } = await inquirer.prompt([
      {
        default: 'router',
        type: 'list',
        message: `Pick the page template for the page`,
        name: 'pageTpl',
        choices: list
      }
    ])
    const targetPagePath = join(cwd, 'page')
    const targetSrcPath = join(cwd, 'src')
    const { __html, __src } = getPageTplPath(pageTpl)
    await fs.copy(__html, join(targetPagePath, pageName + '.html'))
    await fs.copy(__src, join(targetSrcPath, pageName))
    setFireConfig(pageName, true)
    await shell('git add .')
    await delPageTplDir()
    success(`\n √ The page '${pageName}' create completed!`)
  } catch (e) {
    createSpinner.stop()
    await delPageTplDir()
    error(e)
  }
}
