const fs = require('fs-extra')
const { join } = require('path')
const { shellSync } = require('execa')
const { __nodeModules, __fireConfig } = require('./paths')
exports.isFireProject = function() {
  const cwd = process.cwd()
  const { dependencies = {} } = require(join(cwd, 'package.json'))
  return (
    fs.pathExistsSync(join(cwd, 'fire.config.js')) &&
    fs.pathExistsSync(join(cwd, 'src')) &&
    fs.pathExistsSync(join(cwd, 'page')) &&
    dependencies['fire-scripts']
  )
}

exports.setFireConfig = function(name, isFire) {
  if (isFire && !exports.isFireProject()) {
    return false
  }
  if (!fs.pathExistsSync(__fireConfig)) {
    fs.writeFileSync(
      __fireConfig,
      `
module.exports = {
  page: ['${name}']
}
`.trim()
    )
  } else {
    const res = fs.readFileSync(__fireConfig, 'utf8')
    fs.writeFileSync(
      __fireConfig,
      res.replace(
        /page\s*:\s*((\[(.+?)\])|('(.+?)')|("(.+?)"))\s*,?/g,
        `page:['${name}'],`
      )
    )
  }
}
exports.setPkgJson = function(projectName) {
  const latestVer = shellSync('npm view fire-scripts version').stdout
  const { stdout } = shellSync('git config user.name')
  const packagePath = join(process.cwd(), projectName, 'package.json')
  const pkgJson = require(packagePath)
  pkgJson.name = projectName
  pkgJson.author = stdout
  pkgJson.dependencies['fire-scripts'] = latestVer
  fs.writeFileSync(packagePath, JSON.stringify(pkgJson, null, 2))
}
exports.checkFireVersion = function() {
  const currentVer = require(join(__nodeModules, 'fire-scripts/package.json'))
    .version
  try {
    const { stdout } = shellSync('npm view fire-scripts version')
    if (stdout !== currentVer) {
      return {
        currentVer,
        latestVer: stdout
      }
    }
    return false
  } catch (e) {
    return false
  }
}
